import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.server_config_repository_abc import ServerConfigRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.model.server_team_role_ids_config import ServerTeamRoleIdsConfig
from bot_data.model.team_member_type_enum import TeamMemberTypeEnum
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class PermissionService(PermissionServiceABC):
    def __init__(
        self,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        config: ConfigurationABC,
        servers: ServerRepositoryABC,
        server_configs: ServerConfigRepositoryABC,
        technician_configs: TechnicianConfigRepositoryABC,
    ):
        PermissionServiceABC.__init__(self)
        self._logger = logger
        self._bot = bot
        self._config = config
        self._servers = servers
        self._server_configs = server_configs
        self._technician_configs = technician_configs

    def _team_role_members(self, guild_id: int, team_role_type: TeamMemberTypeEnum):
        guild = self._bot.get_guild(guild_id)
        server = self._servers.get_server_by_discord_id(guild_id)
        config = self._server_configs.get_server_config_by_server(server.id)
        admins = []
        for role_id in config.team_role_ids.where(lambda x: x.team_member_type == team_role_type):
            role_id: ServerTeamRoleIdsConfig = role_id
            admins = guild.get_role(role_id.role_id).members

        return admins

    def get_admins(self, g_id: int) -> list[discord.Member]:
        return self._team_role_members(g_id, TeamMemberTypeEnum.admin)

    def get_moderators(self, g_id: int) -> list[discord.Member]:
        return self._team_role_members(g_id, TeamMemberTypeEnum.moderator)

    def is_member_admin(self, member: discord.Member) -> bool:
        return member in self.get_admins(member.guild.id)

    def is_member_moderator(self, member: discord.Member) -> bool:
        return member in self.get_admins(member.guild.id) or member in self.get_moderators(member.guild.id)

    def is_member_technician(self, member: discord.Member) -> bool:
        config = self._technician_configs.get_technician_config()
        return str(member.id) in config.technician_ids.select(lambda x: str(x))
